# Filnavn: microbit-morse.py  
# Morsetraener med Microbit ver 1 og ver 2.
# 35 forskellige morsetegn vises paa displayet
# og afspilles over hoejttaleren 
# knap A = "Repeat"
# knap B = "Next" 
# En morsenoegle kan forbindes mellem GND og P1
# OZ1BXM Lars Petersen, oz1bxm@pobox.com
# December 2020
# Denne kildekode er i public domain
# Credit: HB9ABH Egid Pluess who created 
# morse{} and play_sound(). 

from microbit import * 
import music

tone = 700  	# frekvens (Hz)
speed = 40  	# 40 letters/min
dot = 6000/speed         
dot_length = round(dot)  # convert float to integer
index = 0
pin1.set_pull(pin1.PULL_UP) # P1 is default 1 (pull up)

# This array contains morse letters and numbers
morse = {
  'A': '.-', 'B': '-...', 'C': '-.-.', 'D': '-..', 'E': '.',
  'F': '..-.', 'G': '--.', 'H': '....', 'I': '..', 'J': '.---',
  'K': '-.-', 'L': '.-..', 'M': '--', 'N': '-.', 'O': '---',
  'P': '.--.', 'Q': '--.-', 'R': '.-.', 'S': '...', 'T': '-',
  'U': '..-', 'V': '...-', 'W': '.--', 'X': '-..-', 'Y': '-.--',
  'Z': '--..', '1': '.----', '2': '..---', '3': '...--', '4': '....-',
  '5': '.....', '6': '-....', '7': '--...', '8': '---..', '9': '----.',
  '0': '-----'}

# This array contains the sequence of morse characters
sequence = {
  0: 'A', 1: 'N', 2: 'T', 3: 'E', 4: 'M', 5: 'I', 6: 'K', 7: 'D', 8: 'U', 
  9: 'S', 10: 'R', 11: 'W', 12: 'O', 13: 'L', 14: 'B', 15: 'J', 16: 'V', 
  17: 'P', 18: 'Q', 19: 'X', 20: 'Y', 21: 'C', 22: 'F', 23: 'H', 24: 'Z',
  25: '1', 26: '2', 27: '3', 28: '4', 29: '5', 30: '6', 31: '7', 32: '8', 
  33: '9', 34: '0'}

def s(n):                   # calculate space duration 
    sleep(n * dot_length)
    
def dot():  
    music.pitch(tone, -1)  	# start tone   
    s(1)	  			    # space
    music.stop()           	# stop tone
    s(1)  				    # space

def dash():  
    music.pitch(tone, -1)  	# start tone   
    s(3)  				    # 3 * space
    music.stop()           	# stop tone   
    s(1)  				    # space

def display_led(index): # show character on display
    display.show(sequence[index])
    
def morse_key_status():   # the morse key is connected to P1   
    new_key = pin1.read_digital() # 0 is key-down; 1 is key-up    
    if new_key:                # key-up => no tone     
        music.stop()           
    else:                      
        music.pitch(tone, -1)  # key-down => play tone
        
# remarks concern the first run
def play_sound(index):      # play sound via loudspeaker
    index_copy = sequence[index]     # sequence[0] is "A"  
    dots_dashes = morse[index_copy]  # morse["A"] is ".-" 
    for x in dots_dashes:   # dots_dashes holds 2 characters => loop runs twice 
        if x == '.':        # x is dot => send dot()
            dot()
        else:               # send dash()
            dash()

display_led(index) # show "A" on display
play_sound(index)  # play letter A via loudspeaker
        
while True:           # MAIN LOOP
    morse_key_status()		# maintain morse key

    if button_a.was_pressed():	# "Repeat" button  
        display_led(index) 
        play_sound(index)     
        
    if button_b.was_pressed(): 	# "Next" button 
        index += 1       
        if index == 35:  # range 0-34
            index = 0
        display_led(index) 
        play_sound(index)      